<?php

use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Capsule\Manager as Capsule;

class DeepReplicatorTest extends \PHPUnit_Framework_TestCase
{
    protected $replicator;

    protected function setUp()
    {
        $this->replicator = new \Drp\DeepReplicate\DeepReplicator();

        $this->createDatabase();
    }

    public function createDatabase()
    {
        $capsule = new Capsule;

        $capsule->addConnection([
            'driver'    => 'sqlite',
            'database' => ':memory:'
        ]);

        $capsule->bootEloquent();
        $capsule->setAsGlobal();

        $capsule->schema()->create('humans', function($table)
        {
            $table->increments('id');
            $table->string('name');
            $table->integer('parent_id')->nullable();
        });
    }

    /**
     * @test
     */
    public function deeplyReplicates()
    {
        $parent = Human::create(['name' => 'dad']);
        $child = Human::create(['name' => 'son']);

        $parent->children()->save($child);

        $parentCopy = $this->replicator->deepReplicate($parent);
        $childCopy = $parentCopy->children()->first();

        $this->assertCount(4, Human::all());

        $this->assertNotEquals($parentCopy, $parent);
        $this->assertEquals($parentCopy->name, $parent->name);

        $this->assertNotEquals($childCopy, $child);
        $this->assertEquals($childCopy->name, $child->name);
    }

    /**
     * @test
     */
    public function deeplyReplicatesMultipleLevels()
    {
        $parent = Human::create(['name' => 'dad']);
        $child = Human::create(['name' => 'son']);
        $grandChild = Human::create(['name' => 'grandson']);

        $parent->children()->save($child);
        $child->children()->save($grandChild);

        $parentCopy = $this->replicator->deepReplicate($parent);

        $childCopy = $parentCopy->children()->first();
        $grandChildCopy = $childCopy->children()->first();

        $this->assertCount(6, Human::all());

        $this->assertNotEquals($parentCopy, $parent);
        $this->assertEquals($parentCopy->name, $parent->name);

        $this->assertNotEquals($childCopy, $child);
        $this->assertEquals($childCopy->name, $child->name);

        $this->assertNotEquals($grandChildCopy, $grandChild);
        $this->assertEquals($grandChildCopy->name, $grandChild->name);
    }

    /**
     * @test
     */
    public function avoidsCircularRelations()
    {
        // TODO avoid circular relations
    }
}


class Human extends Model {

    public $timestamps = false;

    protected $fillable = [
        'name'
    ];

    public $replicableRelations = [
        'children'
    ];

    public function parent()
    {
        return $this->hasOne(Human::class, 'parent_id');
    }

    public function children()
    {
        return $this->hasMany(Human::class, 'parent_id');
    }
}